/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - March 20, 2000.
   -
   - The Initial Developer of the Original Code is HJ van Rantwijk.
   - Portions created by HJ van Rantwijk are Copyright (C) 2000-2005
   - HJ van Rantwijk.  All Rights Reserved.
   -
*/

var mzToolbox = null;
var gConfirmForID = -1;
// used in multiviewsUtility.js
var gPopupIconOnTab = true;
var gDelayedPopupIconRemoval = 0;
var gMultiZillaBundle = null;

const mzToolboxVisibilityPrefListener =
{
  domains: [ "multizilla.integrated.toolbar",
             "multizilla.integrated.toolbar.hide-all-other-buttons" ],

  observe: function(subject, topic, aPrefName) {
    if (!mzToolbox || topic != "nsPref:changed")
      return;

    var prefValue = pref.getBoolPref(aPrefName);

    if (aPrefName == this.domains[0]) {
      mzToolbox.mzToolboxSetVisibility(prefValue);
      updateGoogleBoxState(prefValue, false);
    }
    else if (aPrefName == this.domains[1])
      mzToolbox.refreshMultiZillaButtons();
  }
};
const mzTabPrefListener =
{
  domains: [ "multizilla.tabs.mintabwidth", 
             "multizilla.tabs.tabwidth", 
             "multizilla.tabs.status-leds", 
             "multizilla.tabs.title", 
             "multizilla.tabs.cropping", 
             "multizilla.tabs.ontop", 
             "multizilla.tabs.iconclose", 
             "multizilla.tabs.open-order", 
             "multizilla.tabs.focus-order", 
             "multizilla.tabs.meta-refresh",
             "multizilla.tabs.automatic-selection", 
             "multizilla.tabs.label.color", 
             "multizilla.tabs.label.backgroundcolor", 
             "multizilla.tabs.label.selected.color",
             "multizilla.tabs.label.selected.backgroundcolor", 
             "multizilla.tabs.label.selected.usebold" ],

  observe: function(subject, topic, aPrefName) {
    if (!gBrowser || topic != "nsPref:changed")
      return;

    if (aPrefName == this.domains[0])
      gBrowser.setTabWidthTo("minwidth", pref.getIntPref(aPrefName));
    else if (aPrefName == this.domains[1])
      gBrowser.setTabWidthTo("maxwidth", pref.getIntPref(aPrefName));
    else if (aPrefName == this.domains[2])
      gBrowser.setTabLEDstateTo(pref.getBoolPref(aPrefName));
    else if (aPrefName == this.domains[3])
      gBrowser.setTabLabelsTo(pref.getBoolPref(aPrefName));
    else if (aPrefName == this.domains[4])
      gBrowser.setLabelCropping();
    else if (aPrefName == this.domains[5]) {
      if (pref.getBoolPref(aPrefName))
        gBrowser.activateTopTabs();
      else 
        gBrowser.activateBottomTabs();
    }
    else if (aPrefName == this.domains[6])
      gBrowser.setTabCanClose(pref.getBoolPref(aPrefName));
    else if (aPrefName == this.domains[7])
      gBrowser.mTabOrdinance = pref.getIntPref(aPrefName);
    else if (aPrefName == this.domains[8])
      gBrowser.mTabFocus = pref.getIntPref(aPrefName);
    else if (aPrefName == this.domains[9])
      gBrowser.mMetaRefresh = pref.getIntPref(aPrefName);
    else if (aPrefName == this.domains[10])
      gBrowser.mAutomaticTabSelection =pref.getIntPref(aPrefName);
    else if (aPrefName == this.domains[11] || aPrefName == this.domains[12] ||
             aPrefName == this.domains[13] || aPrefName == this.domains[14] ||
             aPrefName == this.domains[15]) {
      gBrowser.setTabLabelColors(aPrefName);
    }
  }
};
const mzTBButtonPrefListener =
{
  domain: "multizilla.showbutton",

  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;
    var buttonName = prefName.substr(this.domain.length+1);
    var buttonId = buttonName + "-tbbutton";
    var button = document.getElementById(buttonId);

    if (!button)
      return;

    var show = pref.getBoolPref(prefName);

    if (show) 
      button.removeAttribute("hidden");
    else 
      button.setAttribute("hidden", "true");
  }
};
const mzTabbarButtonPrefListener =
{
  domains: [ "multizilla.tabbar.newbutton", "multizilla.tabbar.closebutton" ],

  observe: function(subject, topic, prefName) {
    if (!gBrowser || topic != "nsPref:changed")
      return;

    var button = pref.getBoolPref(prefName);

    if (prefName == this.domains[0])
      gBrowser.mTabContainer.setAttribute("displaynew", button ? "true" : "false");
    else if (prefName == this.domains[1])
      gBrowser.mTabContainer.setAttribute("displayclose", button ? "true" : "false");
  }
};
const mzTabIconPrefListener =
{
  domains: [ "browser.chrome.site_icons", "browser.chrome.favicons" ],

  observe: function(subject, topic, prefName) {
    if (!gBrowser || topic != "nsPref:changed")
      return;

    if (prefName == this.domains[0])
      gBrowser.updateDisplayTabIcons();
    else if (prefName == this.domains[1])
      gBrowser.updateDisplayTabIcons();
  }
};
const mzTabAdvancedOpenTabPrefListener =
{
  domains: [ "multizilla.newtabfor.native-applications",
             "advanced.system.supportDDEExec" ],

  observe: function(subject, topic, aPrefName) {
    if (!pref || topic != "nsPref:changed")
      return;

    if (aPrefName == "multizilla.newtabfor.native-applications")
      pref.setBoolPref(this.domains[1], !pref.getBoolPref(aPrefName));
    else if (aPrefName == "advanced.system.supportDDEExec")
      pref.setBoolPref(this.domains[0], !pref.getBoolPref(aPrefName));
  }
};
const mzTabSessionStartupPrefListener =
{
  domains: [ "browser.startup.page",
             "multizilla.browsersession.skip.navigator.startup",
             "multizilla.tabs.restore-session.behavior" ],

  observe: function(subject, topic, aPrefName) {
    if (!pref || topic != "nsPref:changed")
      return;

    var prefValue;

    if (aPrefName == "multizilla.browsersession.skip.navigator.startup") {
      prefValue = pref.getBoolPref(aPrefName);
      if (prefValue)
        pref.setIntPref(this.domains[0], 0);
    }
    else if (aPrefName == "browser.startup.page") {
      prefValue = pref.getIntPref(aPrefName);
      if (prefValue > 0)
        pref.setBoolPref(this.domains[1], false);
    }
    else if (aPrefName == "multizilla.tabs.restore-session.behavior") {
      prefValue = pref.getIntPref(aPrefName);
      if (prefValue > 0)
        pref.setBoolPref(this.domains[1], true);
    }
  }
};
const mzTabPopupIconPrefListener =
{
  domains: [ "multizilla.tabs.display-popup-icon",
             "multizilla.tabs.remove-popup-icon-delay" ],

  observe: function(subject, topic, aPrefName) {
    if (topic != "nsPref:changed")
      return;

    if (aPrefName == this.domains[0])
      gPopupIconOnTab = pref.getBoolPref(aPrefName);
    else if (aPrefName == this.domains[1])
      gDelayedPopupIconRemoval = pref.getIntPref(aPrefName);
  }
};
const mzStatusbarButtonPrefListener =
{
  domains: [ "multizilla.statusbar.display-blocked-windows-button",
             "multizilla.statusbar.display-allowPlugins-button",
             "multizilla.statusbar.display-allowJavascript-button",
             "multizilla.statusbar.display-allowMetaRedirects-button",
             "multizilla.statusbar.display-allowSubframes-button",
             "multizilla.statusbar.display-allowImages-button" ],

  observe: function(subject, topic, aPrefName) {
    if (topic != "nsPref:changed")
      return;

    var prefValue = pref.getBoolPref(aPrefName)
    var id = aPrefName.replace('multizilla.statusbar.display-', '');
    var button = document.getElementById(id);

    if (button)
      button.setAttribute('show', prefValue ? 'true' : 'false');
  }
};
const mzNewWindowRedirectorPrefListener =
{
  domain: "multizilla.window-open.redirector",

  observe: function(subject, topic, prefName) {

    if (!gBrowser || topic != "nsPref:changed")
      return;

    var prefValue = pref.getBoolPref(prefName)
    gBrowser.newWindowRedirector(prefValue);
  }
};
const mzJavaEnabledPrefListener =
{
  domains: [ "security.enable_java",
             "multizilla.content-blocker.java-enabled" ],

  observe: function(subject, topic, prefName) {

    if (topic != "nsPref:changed")
      return;
    if (prefName == this.domains[0])
      pref.setBoolPref(this.domains[1], pref.getBoolPref(this.domains[0]));  
    else if (prefName == this.domains[1])
      pref.setBoolPref(this.domains[0], pref.getBoolPref(this.domains[1]));      
  }
};
//////////////////////////////////////////////////////////////////////////
// Replacement for 'gPopupPrefListener' in navigator.js
//////////////////////////////////////////////////////////////////////////
const mzPopupPrefListener =
{
  domain: "dom.disable_open_during_load",

  observe: function(subject, topic, prefName) {        
    if (!pref || topic != "nsPref:changed")
      return;

    var whitelist = pref.getBoolPref(prefName);

    if (whitelist && pref.getBoolPref("privacy.popups.first_popup"))
      pref.setBoolPref("privacy.popups.first_popup", false);

    var hosts = [];
    var permissionManager = Components.classes["@mozilla.org/permissionmanager;1"]
                                      .getService(Components.interfaces.nsIPermissionManager);
    var enumerator = permissionManager.enumerator;
    var capability = whitelist ? nsIPermissionManager.DENY_ACTION : nsIPermissionManager.ALLOW_ACTION;

    while (enumerator.hasMoreElements()) {
      var nextPermission = enumerator.getNext();
      nextPermission = nextPermission.QueryInterface(Components.interfaces.nsIPermission);
      if (nextPermission.type == "popup") {
        if (nextPermission.capability == capability)
          hosts[nextPermission.host] = nextPermission.host;
      }
    }
    var popupIcon = document.getElementById("popupIcon");
    var browsers = gBrowser.mPanelContainer.childNodes.length;
    
    for (var i = 0; i < browsers; i++) {
      var browser = gBrowser.getBrowserAtIndex(i);

      if (!whitelist && browser.popupDomain in hosts)
        break;

      browser.popupDomain = null;
      popupIcon.hidden = true;
    }
  }
};
//////////////////////////////////////////////////////////////////////////
const gXPInstallObserver =
{
  _multizillaBundle : srGetStrBundle("chrome://multiviews/locale/multizilla.properties"),
  _blockedXPInstallPrefString : "multizilla.blocked-xpinstall.sound_enabled",
  _blockedXPInstallSoundPrefString : "multizilla.blocked-xpinstall.sound_url",
  _XPInstallImageURL : "chrome://multiviews/content/images/managers/xpinstall-small.png",
  _showCloseButtonPrefString : "multizilla.info-messages.show-close-botton",

  findChildShell: function(aDocShell, aSoughtShell)
  {
    if (aDocShell == aSoughtShell)
      return aDocShell;
      
    var node = aDocShell.QueryInterface(Components.interfaces.nsIDocShellTreeNode);
    for (var i = 0; i < node.childCount; ++i) {
      var docShell = node.getChildAt(i);
      docShell = this.findChildShell(docShell, aSoughtShell);
      if (docShell == aSoughtShell)
        return docShell;
    }
    return null;
  },
  
  getBrowser: function (aDocShell) 
  {
    var browsers = gBrowser.mPanelContainer.childNodes.length;

    for (var i = 0; i < browsers; ++i) {
      var browser = gBrowser.getBrowserAtIndex(i);
      var soughtShell = aDocShell;
      var shell = this.findChildShell(browser.docShell, soughtShell);
      if (shell)
        return browser;
    }
    return null;
  },

  initPopup: function (aEvent)
  {
    dump("\ninitPopup();");
  },

  observe: function(aSubject, aTopic, aData)
  {
    var browser = null;
    var webNav = null;

    switch (aTopic)
    {
      case "xpinstall-install-blocked":
        dump("\nxpinstall-install-blocked");
        var shell = aSubject.QueryInterface(Components.interfaces.nsIDocShell);
        browser = this.getBrowser(shell);

        if (browser) {
          var currentURI = browser.docShell.QueryInterface(Components.interfaces.nsIWebNavigation).currentURI;
          var host = (currentURI.scheme == "file") ? "file:" : currentURI.hostPort;
          var messageString = "";
          var buttonString = "";
          var topBottom = gBrowser.mTopTabs ? "top" : "bottom";
          var showCloseButton = mzPreferences.readMyPref(this._showCloseButtonPrefString, "bool", true);
		
          if (!mzPreferences.readMyPref("xpinstall.enabled", "bool", true))
            messageString = this._multizillaBundle.formatStringFromName("xpinstallDisabledWarning", [host], 1);
          else
            messageString = this._multizillaBundle.formatStringFromName("xpinstallWarning", [host], 1);

          gBrowser.showInfoMessage(browser, this._XPInstallImageURL, messageString, "", null, null, "XPInstallPopupOptions", topBottom, showCloseButton);
        }
        if (mzPreferences.readMyPref(this._blockedXPInstallPrefString, "bool", true))
          mzPlaySound(mzPreferences.readMyPref(this._blockedXPInstallSoundPrefString, "string", ""));
        break;

      case "xpinstall-install-edit-prefs":
        dump("\nxpinstall-install-edit-prefs");
        mzManagers.openPermissionManager("installManager-Preferences");
        gBrowser.hideInfoMessage(gBrowser.selectedBrowser);
        break;

      case "xpinstall-install-edit-permissions":
        dump("\nxpinstall-install-edit-permissions");
        browser = this._getBrowser(aSubject.QueryInterface(Components.interfaces.nsIDocShell));

        if (browser) {
          webNav = aSubject.QueryInterface(Components.interfaces.nsIWebNavigation);
          mzManagers.openPermissionManager("installManager", webNav.currentURI.host);
        }
        gBrowser.hideInfoMessage(gBrowser.selectedBrowser);
        break;
    }
  }
};
////////////////////////////////////////////////////////////////////////////////
function multiZillaInit()
{
  var doInitialize = true;

  if ('arguments' in window && window.arguments.length == 2 && window.arguments[1].indexOf("turbo=yes") != -1)
    doInitialize = false;
  if (doInitialize) {
    mzAddPrefListener(mzToolboxVisibilityPrefListener);
    mzAddPrefListener(mzTabPrefListener);
    mzAddPrefListener(mzTBButtonPrefListener);
    mzAddPrefListener(mzTabbarButtonPrefListener);
    mzAddPrefListener(mzTabIconPrefListener);
    mzAddPrefListener(mzTabAdvancedOpenTabPrefListener);
    mzAddPrefListener(mzTabSessionStartupPrefListener);
    mzAddPrefListener(mzTabPopupIconPrefListener);
    mzAddPrefListener(mzStatusbarButtonPrefListener);
    mzAddPrefListener(mzNewWindowRedirectorPrefListener);
    mzAddPrefListener(mzJavaEnabledPrefListener);

    var _nsIObserverService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    _nsIObserverService.addObserver(gXPInstallObserver, "xpinstall-install-blocked", false);
    _nsIObserverService.addObserver(gXPInstallObserver, "xpinstall-install-edit-prefs", false);
    _nsIObserverService.addObserver(gXPInstallObserver, "xpinstall-install-edit-permissions", false);
  }
  mzPrefInit.initPreferences();
  initializeRDFStorageFile();
}
////////////////////////////////////////////////////////////////////////////////
function mzShutdown()
{
  mzRemovePrefListener(mzJavaEnabledPrefListener);
  mzRemovePrefListener(mzNewWindowRedirectorPrefListener);
  mzRemovePrefListener(mzStatusbarButtonPrefListener);
  mzRemovePrefListener(mzTabPopupIconPrefListener);
  mzRemovePrefListener(mzTabSessionStartupPrefListener);
  mzRemovePrefListener(mzTabAdvancedOpenTabPrefListener);
  mzRemovePrefListener(mzTabIconPrefListener);
  mzRemovePrefListener(mzTabbarButtonPrefListener);
  mzRemovePrefListener(mzTBButtonPrefListener);
  mzRemovePrefListener(mzTabPrefListener);
  mzRemovePrefListener(mzToolboxVisibilityPrefListener);

  var _nsIObserverService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  _nsIObserverService.removeObserver(gXPInstallObserver, "xpinstall-install-blocked");
  _nsIObserverService.removeObserver(gXPInstallObserver, "xpinstall-install-edit-prefs");
  _nsIObserverService.removeObserver(gXPInstallObserver, "xpinstall-install-edit-permissions");

  Shutdown(); // navigator.xul
}
////////////////////////////////////////////////////////////////////////////////
// this function sets ID's to the Go Menu and replaces the popup
// I know this is a dirty way to do it, but we need to ;)
////////////////////////////////////////////////////////////////////////////////
function addGoMenuIDs()
{
  var mainMenu = document.getElementById("main-menubar");
  var goMenu = mainMenu.childNodes[3];
  goMenu.setAttribute("id", "menu_Go");
  goMenu.firstChild.setAttribute("id", "menu_GoPopup");
  var goMenuPopup = document.getElementById("newMenu_GoPopup");
  // We can't use replaceChild because that is broken in mozilla!!!
  // goMenu.replaceChild(goMenuPopup, goMenu.firstChild);
  goMenu.insertBefore(goMenuPopup, goMenu.firstChild);
  goMenu.removeChild(goMenu.lastChild);
  // Replace default image manager with MultiZilla's Permission Manager
  var menuitem = document.getElementById('AllowImages');
  menuitem.nextSibling.nextSibling.setAttribute("oncommand", "mzManagers.openPermissionManager('imageManager');");
  // Replace default cookie manager with MultiZilla's Permission Manager
  menuitem = document.getElementById('AllowCookies');
  menuitem.nextSibling.nextSibling.setAttribute("oncommand", "mzManagers.openPermissionManager('cookieManagerFromIcon');");

  var prefIdentifier = "multizilla.statusbar.display-";
  var blockedWindowButtonID = "blocked-windows-button";
  var prefValue = mzPreferences.readMyPref(prefIdentifier + blockedWindowButtonID, "bool", true);
  var button = document.getElementById(blockedWindowButtonID);
  button.setAttribute("show", prefValue ? "true" : "false");

  for (index in gBrowser.mDocShellPropertyNames) {
    var property = gBrowser.mDocShellPropertyNames[index];
    prefValue = mzPreferences.readMyPref((prefIdentifier + property + "-button"), "bool", true);
    button = document.getElementById(property + "-button");
    button.setAttribute("show", prefValue ? "true" : "false");
  }
  // document.documentElement.setAttribute("classicStyle", "true");
}
////////////////////////////////////////////////////////////////////////////////
function initBookmarkTemplates()
{
  initializeBookmarksTemplate();
  initializePersonalToolbarTemplate();

  // select modified template for the Bookmarks menu
  var bookmarksMenu = document.getElementById("BookmarksMenu");
  bookmarksMenu.setAttribute("template", "BookmarksMenuTemplate");
  // select modified template for the Bookmarks Button
  var bookmarksButton = document.getElementById("bookmarks-button");
  bookmarksButton.setAttribute("template", "BookmarksButtonTemplate");
  // select modified template for the Bookmarks Overflow Menu
  var bookmarksChevron = document.getElementById("bookmarks-chevron");
  bookmarksChevron.setAttribute("template", "BookmarksButtonTemplate");
  // replace default PT template with our modified version
  var originalPTBox = document.getElementById("bookmarks-ptf");
  var newPTBox = document.getElementById("NEW-NC:PersonalToolbarFolder");
  var clonedTemplate = newPTBox.childNodes[1].cloneNode(true);
  originalPTBox.replaceChild(clonedTemplate, originalPTBox.childNodes[1]);

  if (!/Mac/.test(navigator.platform)) { // Modify Main Bookmarks menu
    // bookmarksMenu.removeAttribute("oncommand"); XXX: see bug 3324
    // bookmarksMenu.setAttribute("onclick", "OpenBookmarkURL(event.target, this.database, event)");
  }
  // cleanup: remove dummy bookmarks menu item
  var dummyBookmarkPopup = document.getElementById("dummyBookmarkPopup");
  dummyBookmarkPopup.removeChild(newPTBox);

  // theme support: added for Mac(OSX)Zilla support
  bookmarksButton.firstChild.setAttribute("class", "bookmarks-menupopup");
  bookmarksChevron.firstChild.setAttribute("class", "bookmarks-menupopup");
}
////////////////////////////////////////////////////////////////////////////////
function initInstalledToolbox()
{
  // GoogleBox support: added for MultiZilla's GoogleBox
  mzToolbox = document.getElementById("MultiZillaToolbar");
  mzToolbox.updateButtons("multizilla.showbutton.");
  var toolboxEnabled = mzPreferences.readMyPref("multizilla.integrated.toolbar", "bool", false);
  mzToolbox.mzToolboxSetVisibility(toolboxEnabled);
  var showInternalToolbox = document.getElementById("cmd_viewIntegratedToolBox");

  if (toolboxEnabled) {
    if (showInternalToolbox.getAttribute("checked") == "true") {
      document.getElementById("internalToolBox").removeAttribute("hidden");
      mzToolbox.refreshMultiZillaButtons();
    }
  }
  else {
    showInternalToolbox.setAttribute("checked", "false");
    setTimeout(toggleOrEnableGoogleBox, 2500, true);
  }
}
////////////////////////////////////////////////////////////////////////////////
function initOrdinals()
{
  var nodeList = document.getAnonymousNodes(getBrowser().mTabContainer);
  var isMac = /Mac/.test(navigator.platform);
  var isMacClassic = (isMac && nodeList.length == 1);

  if (isMacClassic) {
    // http://lxr.mozilla.org/seamonkey/source/themes/classic/global/mac/globalBindings.xml
    nodeList = nodeList[0].childNodes[1].firstChild.childNodes;
  }
  for (var i = 0; i < nodeList.length; i++) {
    nodeList[i].ordinal = i;
    if (nodeList[i].localName == "hbox") {
      nodeList[i].setAttribute("class", "multizilla-tabs");
      nodeList[i].firstChild.ordinal = 0;
      nodeList[i].lastChild.ordinal = 9999999;
    }
  }
  if (nodeList.length > 1)
    nodeList[nodeList.length-(1+isMac)].ordinal = 9999999;
  gBrowser.mTabContainer.advanceSelectedTab = advanceSelectedTab;
}
////////////////////////////////////////////////////////////////////////////////
function initEnhancedPopupBlocker()
{
  // Remove default popup pref listener, because it fails to work properly!
  removePrefListener(gPopupPrefListener);
  // Use a 'heavily' modified copy of it
  addPrefListener(mzPopupPrefListener);
  /****
  * We need to get notified of blocked popup windows so we replace the 
  * original eventlistener for 'DOMPopupBlocked' with our (this) event 
  * listener and call 'onPopupBlocked' (original function) when we're done.
  */
  addEventListener("DOMPopupBlocked", mzOnPopupBlocked, false);
  // removeEventListener("DOMPopupBlocked", onPopupBlocked, false);
  gPopupIconOnTab = mzPreferences.readMyPref("multizilla.tabs.display-popup-icon", "bool", true);
  gDelayedPopupIconRemoval = mzPreferences.readMyPref("multizilla.tabs.remove-popup-icon-delay", "int", 0);

  window.document.addEventListener("MZScriptBlocked", mzOnScriptBlocked, false);
  window.document.addEventListener("MZImageBlocked", mzOnImageBlocked, false);
  window.document.addEventListener("MZObjectBlocked", mzObjectBlocked, false);
  window.document.addEventListener("MZFrameBlocked", mzOnFrameBlocked, false);
  // window.document.addEventListener("MZMetaRedirectBlocked", mzMetaRedirectBlocked, false);
  window.document.addEventListener("MZDocumentBlocked", mzOnDocumentBlocked, false);
}
////////////////////////////////////////////////////////////////////////////////
function mzGetBundle(aBundleName)
{
  aBundleName = aBundleName ? "chrome://multiviews/locale/" + aBundleName :
                              "chrome://multiviews/locale/multizilla.properties";
  return (aBundleName ? srGetStrBundle(aBundleName) : null);
}
////////////////////////////////////////////////////////////////////////////////
// aStr = string, aParam = array of params used to replace the %S arguments
// (you need one param for each %S) and aBundleName = filename.properties
////////////////////////////////////////////////////////////////////////////////
function mzGetTextFromBundle(aStr, aParam, aBundleName)
{
  var aBundle = mzGetBundle(aBundleName);
  if (!aBundle) {
    alert("aBundle error!");
    return "error";
  }
  try {
    var arguments = (aParam) ? aParam.length : 1;
    var text = aParam ? aBundle.formatStringFromName(aStr, aParam, arguments) : 
                        aBundle.GetStringFromName(aStr);
  } 
  catch(ex) {
    alert("Error: '"+aStr+"' not found in "+(aBundleName ? aBundleName : "multizilla.properties") +
          (aParam ? "\n\nNote: 'aParam' is specified, make sure to use an [array]" : ""));
  }
  return text;
}
////////////////////////////////////////////////////////////////////////////////
// Added for compatibility with pre 1.5 mozilla builds (20032406 and later)
////////////////////////////////////////////////////////////////////////////////
function checkConfirmCloseWS(aConfirm)
{
  // Give 'tryToClose' (WindowIsClosing()) a chance to veto if it is defined
  if (typeof(window.tryToClose) == "function") {
    gConfirmForID = aConfirm;
    if (aConfirm == 4) { // onclose="checkConfirmCloseWS(4)" in multizillaOverlay.xul
      return WindowIsClosing();
    }
  }
  else {
    return confirmCloseWS(aConfirm);
  }
  return true;
}
////////////////////////////////////////////////////////////////////////////////
function confirmCloseWS(aConfirm)
{
  /* ::::
     1  = File Menu 'Close Window' & Ctrl-Shift-W
     2  = File Menu 'Exit' & Ctrl-Q
     4  = Click on [X] on Window
     8  = JavaScript close()
     16 = no confirmation for single tab
  :::: */
  if (aConfirm == 2) {
    var lastChoice = mzPreferences.readMyPref("multizilla.confirm.quit-session", "bool", false);
    if (lastChoice) {
      saveTabSession(false, true);
      return true;
    }
  }
  var confirmPref = mzPreferences.readMyPref("multizilla.confirm.quit", "int", 11);
  var aHeader = mzGetTextFromBundle("closeSession");
  var aQuestion = mzGetTextFromBundle("closeSessionQuestion");
  var numberOfTabs = gBrowser.mTabContainer.childNodes.length;

  if (numberOfTabs == 1 && confirmPref & 16) {
    saveTabSession(false, true);
    return true;
  }
  else if (aConfirm == 8 && document.documentElement.hasAttribute("chromehidden")) {
    saveTabSession(false, true);
    return true;
  }
  else if (confirmPref && (confirmPref & aConfirm) == aConfirm) {
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);

    if (aConfirm & 1 || aConfirm & 4) {
      aHeader = mzGetTextFromBundle("closeWindow");
      aQuestion = mzGetTextFromBundle("closeWindowQuestion");
    }
    if (promptService.confirm(window, aHeader, aQuestion)) {
      if (aConfirm == 2) { // Close session confirmed, remember this, write pref!
        mzPreferences.writePref("bool", "multizilla.confirm.quit-session", true);
      }
      saveTabSession(false, true);
      return true;
    }
    else {
      if (aConfirm == 2) { // Close session, clear pref
        mzPreferences.writePref("bool", "multizilla.confirm.quit-session", false);
      }
      return false;
    }
  }
  saveTabSession(false, true);
  return true;
}

var firstBrowserWindow = null;
////////////////////////////////////////////////////////////////////////////////
function isFirstWindow(aType)
{
  var defaultType = "navigator:browser";

  if (!aType) // default check for browser window
    aType = defaultType;

  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var topWindow = windowManagerInterface.getMostRecentWindow(aType);
  var enumerator = windowManagerInterface.getEnumerator(aType);
  var firstWindow = enumerator.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
  var windowType = topWindow.document.documentElement.getAttribute("windowtype")
  // Initialize 'firstBrowserWindow' the first time only!
  if (firstBrowserWindow == null && windowType == defaultType)
    firstBrowserWindow = firstWindow; // this is used in mzWindowInit()

  if (firstWindow != topWindow)
    return false;
  return true;
}
////////////////////////////////////////////////////////////////////////////////
function getNumberOfWindowsByType(aType)
{
  var defaultType = "navigator:browser";

  if (!aType) // default check for browser window
    aType = defaultType;

  var windowCounter = 0;
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
  var enumerator = windowManager.getEnumerator(aType);

  while (enumerator.hasMoreElements()) {
    var aWindow = enumerator.getNext();
    windowCounter++;
  }
  return windowCounter;
}
////////////////////////////////////////////////////////////////////////////////
function isLastWindow(aType)
{
  return (getNumberOfWindowsByType(aType) == 1);
}
////////////////////////////////////////////////////////////////////////////////
// This function is used in multiviewsContext.js
////////////////////////////////////////////////////////////////////////////////
function isNavigatorWindow(aWindow) {
  var win = window.top;
  if (aWindow)
    win = aWindow;
  return (win.document.documentElement.getAttribute('windowtype') == 'navigator:browser');
}
////////////////////////////////////////////////////////////////////////////////
function getMostRecentWindow(aType, aChromeCheckFlag)
{
  var defaultType = "navigator:browser";

  if (!aType)
    aType = defaultType;

  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var navigatorWindow = windowManagerInterface.getMostRecentWindow(aType);
  // Prevent opening of new tabs in a chromeless window!
  if (aChromeCheckFlag && aType == "navigator:browser") {
    var enumerator = windowManager.getEnumerator(aType);
    var windowArray = new Array();

    while (enumerator.hasMoreElements()) {
      var win = enumerator.getNext();

      if (!win.document.documentElement.hasAttribute("chromehidden"))
        windowArray.push(win);
    }
    return windowArray[windowArray.length-1];
  }
  return navigatorWindow;
}
////////////////////////////////////////////////////////////////////////////////
function getMostRecentTabbrowser()
{
  var win = getMostRecentWindow();
  return win ? getMostRecentWindow().getBrowser() : null;
}
////////////////////////////////////////////////////////////////////////////////
// This function modifies the default mozilla startup.
////////////////////////////////////////////////////////////////////////////////
function mzWindowInit()
{
  var newTab = null;
  var externalLinks = mzPreferences.readMyPref("multizilla.newtabfor.external-links", "bool", true);
  var nativeApplications = mzPreferences.readMyPref("multizilla.newtabfor.native-applications", "bool", true);
  var doCheck = false;
  var URL;
  var winArgs = false;

  if ('arguments' in window) {
    URL = window.arguments[0];
    winArgs = window.arguments.length;
  }

  if ((externalLinks || nativeApplications) && winArgs && window.opener == null) {
    if (URL != "" && URL != null && URL != "about:blank") {
      if (!isFirstWindow() && firstBrowserWindow != null) {
        var tabbrowser = firstBrowserWindow.getBrowser();
        firstBrowserWindow.focus();
        if (mzPreferences.readMyPref("multizilla.tabs.blank-tab-first", "bool", true)) {
          newTab = checkForBlankTab(tabbrowser);
          if (newTab != null)
            newTab.loadURI(URL);
        }
        if (newTab == null) { // Add new tab with URL. Pref not set or no blank tab available
          newTab = tabbrowser.addTab(URL);
          tabbrowser.selectedTab = newTab; // Don't we need a pref setting for this?
        }
        window.document.documentElement.setAttribute("onunload", "");
        window.close();
        return false;
      }
      // This is, well at least it should be, the first navigator:browser window. 
      else
        doCheck = true;
    }
  }
  if (doCheck || isFirstWindow()) {
    // Sanity check (mozilla crashed before writing our pref setting?).
    mzResetConfirmationPref();
    // Do we need to restore a previously saved browser session?
    if (mzPreferences.readMyPref("multizilla.tabs.restore-session.behavior", "int", 3) > 0) {
      var startup = pref.getIntPref("browser.startup.page");
      // Browser sessions will only be restored in limited cases
      if (startup == 0 && URL == "about:blank")
        getBrowser().mRestoreBrowserSession = true;
      else if (startup == 1 && URL == getHomePage())
        getBrowser().mRestoreBrowserSession = true;
      else if (startup == 2) {
        var globalHistory;
        // Builds 20040210 and up (see also mozilla bug 224829) 
        if ("@mozilla.org/browser/global-history;2" in Components.classes)
          globalHistory = Components.classes["@mozilla.org/browser/global-history;2"];
        else 
          globalHistory = Components.classes["@mozilla.org/browser/global-history;1"];

        globalHistory = globalHistory.getService(Components.interfaces.nsIBrowserHistory);

        if (URL == globalHistory.lastPageVisited)
          getBrowser().mRestoreBrowserSession = true;
      }
    }
  }
  Startup();
  if (!getBrowser().mInit)
    getBrowser().enterTabbedMode();
  return true;
}
////////////////////////////////////////////////////////////////////////////////
// Replacement for mozilla's close window prompt in navigator.js
////////////////////////////////////////////////////////////////////////////////
function WindowIsClosing()
{
  var shouldClose = true;
  var browser = getBrowser();

  if (!browser.mChromeHidden) {
    var confirmForID = (gConfirmForID != -1) ? gConfirmForID : 2; // -1 is Composer, Mail, Addressbook or the JS Console
    shouldClose = confirmCloseWS(confirmForID);

    if (shouldClose && isLastWindow()) {
      mzResetConfirmationPref();
      mzCleanupOnExit();

      if (mzPreferences.readMyPref("multizilla.uam.set-default-useragent-onbrowserexit", "bool", true)) {
        var uaPrefIdentifier = "general.useragent.override";

        if (pref.prefHasUserValue(uaPrefIdentifier)) {
          pref.setCharPref(uaPrefIdentifier, "");
          pref.clearUserPref(uaPrefIdentifier);
        }
      }
    }
  }
  return shouldClose;
}
////////////////////////////////////////////////////////////////////////////////
function mzCleanupOnExit()
{
  var list = "";
  var prefIdentifier = ["diskcache", "locationbar", "history", "cookies", "downloads", "formdata", "passwords"];
  // Initialize list based on pref values
  for (i in prefIdentifier) {
    if (mzPreferences.readMyPref("multizilla.privacy.clear-on-exit." + prefIdentifier[i], "bool", false))
      list = list ? list + ", " + prefIdentifier[i] : prefIdentifier[i];
  }
  if (list.length > 0) { // Do we have anything to remove?
    var result = mzPreferences.readMyPref("multizilla.privacy.clear-on-exit.confirm", "bool", true);

    if (result) {
      var checkbox = {value:0};
      var checkboxMsg = mzGetTextFromBundle("clearOnExitDontAskAgain");
      var title = mzGetTextFromBundle("clearOnExitTitle");
      var text = mzGetTextFromBundle("clearOnExitText");
      text = text + list; // Add list
      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
      result = promptService.confirmEx(window, title, text, 
                                       (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES) +
                                       (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_NO),
                                       null, null, null, null, checkbox);
    }
    if (result == 0) {
      var newPrefIdentifier = list.split(", ");

      for (i in newPrefIdentifier) {
        if (newPrefIdentifier[i] == prefIdentifier[0])
          mzPrivacyTools.clearCache("ALL");
        if (newPrefIdentifier[i] == prefIdentifier[1])
          mzPrivacyTools.clearUrlBarHistory()
        if (newPrefIdentifier[i] == prefIdentifier[2])
          mzPrivacyTools.clearGlobalHistory();
        if (newPrefIdentifier[i] == prefIdentifier[3])
          mzPrivacyTools.clearAllCookies();
        if (newPrefIdentifier[i] == prefIdentifier[4])
          mzPrivacyTools.clearAllDownloads();
        if (newPrefIdentifier[i] == prefIdentifier[5])
          mzPrivacyTools.clearAllFormData();
        if (newPrefIdentifier[i] == prefIdentifier[6])
          mzPrivacyTools.clearAllPasswords();
      }
    }
  }        
}
////////////////////////////////////////////////////////////////////////////////
// Called from function 'WindowIsClosing()' and function 'mzWindowInit()'.
////////////////////////////////////////////////////////////////////////////////
function mzResetConfirmationPref()
{
  /***
    * This pref setting will be set to 'true' in function confirmCloseWS()
    * after the user confirms to close the this session. Other open windows 
    * will check this pref setting, to enable a more relaxed/quiet browser 
    * exit, with only a single confirmation asked for close session!
    */
  mzPreferences.writePref("bool", "multizilla.confirm.quit-session", false);
}
////////////////////////////////////////////////////////////////////////////////
function mzConfirmWindowOpen(aHostPort)
{
  var title = mzGetTextFromBundle('newWindowRedirectorTitle'); // 'mzWizard');
  var text = mzGetTextFromBundle('confirmWindowOpenText', [aHostPort], null);
  var selectionPrefValue = mzPreferences.readMyPref("multizilla.window-open.selection.behavior", "int", 0);
  var checkbox = {value:(selectionPrefValue == 2)};
  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
  var windowManagerDS = Components.classes['@mozilla.org/rdf/datasource;1?name=window-mediator'].getService(nsIWindowDataSource);
  var win = windowManagerDS.getWindowForResource(getWindowResourceID(window));
  // returns 0 for [Tab] 1 for [Block] or (ESC) and 2 [Window] button
  var result = promptService.confirmEx(win, title, text, 
                                      (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_IS_STRING) +
                                      (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_IS_STRING) +
                                      (promptService.BUTTON_POS_2 * promptService.BUTTON_TITLE_IS_STRING),
                                      mzGetTextFromBundle('openInTabButton'), 
                                      mzGetTextFromBundle('blockWindowButton'), 
                                      mzGetTextFromBundle('openInWindowButton'), 
									  mzGetTextFromBundle('updatePermissionsCheckbox'), checkbox);
  // Tab Button = 0, Window Button = 2 and Block Button/ESC/Window X = 1
  // Tab Button returns 3, Window Button returns 1 and Block Button/ESC/Window X returns 2
  result = ((result == 0) ? 3 : (result == 1) ? 2 : 1);
  result += (checkbox.value ? 4 : 0);
  return result;
}
////////////////////////////////////////////////////////////////////////////////
function mzWindowOpen(aURL, aName, aFlags)
{
  var newWin = null;
  var opener = new XPCNativeWrapper(window, "opener").opener;

  if (isNavigatorWindow() && opener == null) {
    var contextWindow = window.document.defaultView;
    var sourceURL = new XPCNativeWrapper(contextWindow, "location").location.href;

    if (!aURL.match(/^\w+:/))
      aURL = makeURLAbsolute(sourceURL, aURL);

    const nsIScriptSecurityManager = Components.interfaces.nsIScriptSecurityManager;
    var secMan = Components.classes["@mozilla.org/scriptsecuritymanager;1"].getService(nsIScriptSecurityManager);
    try {
      secMan.checkLoadURIStr(sourceURL, aURL, nsIScriptSecurityManager.STANDARD);
    } catch(ex) {
      throw "Load of " + aURL + " denied.";
    }
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    var sourceURI = ioService.newURI(sourceURL, null, null);
    var targetURI = ioService.newURI(aURL, null, null);
    var browser = aName ? getNamedBrowser(aName) : null;

    if (browser) {
      newWin = browser.contentDocument.defaultView;
      // newWin.opener = contextWindow;
      browser.loadURI(aURL, sourceURI); // sourceURI is used as referrer
      getBrowser().selectedTab = getBrowser().getTabForBrowser(browser);
      return newWin;
    }
    var permissionmanager = Components.classes["@mozilla.org/permissionmanager;1"].getService(Components.interfaces.nsIPermissionManager);
    var capability = permissionmanager.testPermission(sourceURI, "popup");
    dump("\ncapability-1: " + capability);
    if (capability == nsIPermissionManager.UNKNOWN_ACTION) // 0
      capability = mzPreferences.readMyPref("multizilla.window-open.behavior", "int", 2);
    dump("\ncapability-2: " + capability);
    var hostPort;

    try {
      hostPort = (sourceURI.spec.match(/^file:/i)) ? "file:" : sourceURI.hostPort;
    } catch(ex) { 
      dump("\nError getting hostPort: " + sourceURI.spec);
      hostPort = sourceURI.spec;
    }
    var tab = null;

    if ('mouseClick' in contextWindow && contextWindow.mouseClick) {
      var sourceElement = contextWindow.mouseClick;

      if (sourceElement.localName == "browser") { 
        dump("\nmouseClick is a 'browser', allow new window");
        browser = sourceElement;
        tab = gBrowser.getTabForBrowser(browser);
        sourceElement = undefined;
      }
      else if (sourceElement.localName == "browsermessage") {
        dump("\nOpen from a Browser Info Message, allow new window");
        browser = gBrowser.mTopTabs ? sourceElement.nextSibling : sourceElement.previousSibling;
        tab = gBrowser.getTabForBrowser(browser);
        sourceElement = undefined;

        if (capability == nsIPermissionManager.DENY_ACTION)
          capability = nsIPermissionManager.ALLOW_ACTION;
      }
      else if (contextWindow.mouseClick == false) {
        dump("\nmouseClick is 'false', block new window!");
        capability = nsIPermissionManager.DENY_ACTION;
      }
      else if (contextWindow.mouseClick == undefined) {
        dump("\nmouseClick is 'undefined', allow new window!");
        capability = nsIPermissionManager.ALLOW_ACTION;
      }
    }
    else {
      // Check permissions for targetURI
      // if (permissionmanager.testPermission(sourceURI, "popup") == nsIPermissionManager.ALLOW_ACTION) {
        var targetPM = permissionmanager.testPermission(targetURI, "popup");
        dump("\ntargetPM: " + targetPM);
        if (targetPM == nsIPermissionManager.UNKNOWN_ACTION)
          capability = mzPreferences.readMyPref("multizilla.window-open.behavior", "int", 2);
        else 
          capability = targetPM;
      /* }
      else 
        capability = nsIPermissionManager.DENY_ACTION; */
    }
    if ('clickedOnJSLink' in contextWindow) { // clicked on a javascript: link!
      if (mzPreferences.readMyPref("multizilla.window-open.javascript-in-tab", "bool", false)) {
        if (contextWindow.clickedOnJSLink)
          capability = 3;
      }
      else if (contextWindow.clickedOnJSLink && contextWindow.clickedOnJSLink.localName == "browser") {
        dump("\nclickedOnJSLink is a 'browser', open in a new tab?");
        browser = contextWindow.clickedOnJSLink;
        tab = gBrowser.getTabForBrowser(browser);
      }
      contextWindow.clickedOnJSLink = undefined;
    }
    dump("\ncapability: " + capability);
    if (capability == 4) { // Open dialog and ask what he want to do with this request
      dump("\nif (capability == 4)");
      var previouslyBlocked = isHostBlocked(getBrowserForDocument(contextWindow.document), hostPort);
      var selectionPrefValue = mzPreferences.readMyPref("multizilla.window-open.selection.behavior", "int", 0);

      if (previouslyBlocked && selectionPrefValue == 1) {
        capability = nsIPermissionManager.DENY_ACTION;
      }
      else {
        if (tab)
          tab.setAttribute("dialog", "true");

        capability = mzConfirmWindowOpen(hostPort);

        if (tab)
          tab.removeAttribute("dialog");

        if (capability & 4) {
          capability -= 4;
          permissionmanager.add(sourceURI, "popup", capability);
        }
      }
    }
    var charset = 'charset=' + window.document.characterSet;
    aName = aName ? aName : '_blank';

    if (capability == 3) { // Redirect this window to a new tab
      dump("\nREDIRECT_ACTION");
      tab = preventDupeTab(aURL, false, getBrowser());

      if (!tab)
        tab = getBrowser().addTab(aURL, sourceURI, charset, null, aName); // sourceURI is used as referrer
      browser = getBrowser().getBrowserForTab(tab);
      return browser.contentDocument.defaultView;
    }
    else if (capability == nsIPermissionManager.DENY_ACTION) { // 2
      dump("\nDENY_ACTION");
      // Construct popup object with the window properties
      var aPopupObject = new Object();
      aPopupObject.requestingWindowURI = sourceURI;
      aPopupObject.hostPort = hostPort;
      aPopupObject.popupWindowName = aName;
      aPopupObject.popupWindowURI = targetURI;
      aPopupObject.popupWindowFeatures = aFlags ? aFlags.replace(/(scrollbars|resizable|minimizable|close)(=(0|false|no))?,?/ig, '') : "";
      dump("\npopupWindowFeatures-1: " + aPopupObject.popupWindowFeatures);
      /***
        * This event triggers mzOnPopupBlocked() and that function adds the URL to the 
        * blocked popup window menu (right side of the statusbar) and calls mozilla's 
        * onPopupBlocked() to play the popup sound
        */
      var URI = ioService.newURI(aURL, null, null);
      event = document.createEvent('PopupBlockedEvents');
      event.initPopupBlockedEvent('DOMPopupBlocked', true, true, sourceURI, URI, aPopupObject.popupWindowFeatures);
      contextWindow.top.document.dispatchEvent(event);
      return null; // This might not be right. I'll need to find out what to do at this point
    }
    else if (capability == nsIPermissionManager.ALLOW_ACTION) { // 1
      dump("\nALLOW_ACTION");
      if (mzPreferences.readMyPref("multizilla.window-open.override-features", "bool", false) == true) { 
        var regexp = new RegExp();
        var newWinFeatures = aFlags || "";
        var windowFeatures = [ "chrome", "extrachrome", "titlebar", "menubar", 
                               "toolbar", "location", "personalbar", "directories", 
                               "minimizable", "resizable", "close", "scrollbars", "status"];

        var overrideValue = mzPreferences.readMyPref("multizilla.window-open.features", "int", 2047);
        // Check pref setting, but prevent jokers from disabling chrome and extrachrome!
        for (var i in windowFeatures) {
          if (i < 2 || overrideValue & (1 << i)) {
            newWinFeatures = newWinFeatures.replace(regexp.compile(windowFeatures[i]+'=(0|no|false)', 'ig'), windowFeatures[i]);
            if (newWinFeatures.indexOf(windowFeatures[i]) < 0)
              newWinFeatures = newWinFeatures ? newWinFeatures + ',' + windowFeatures[i] : windowFeatures[i];
          }
        }
        var win = this.openDialog(getBrowserURL(), aName, newWinFeatures, aURL, charset, sourceURI); // sourceURI is used as referrer
        // win = Object ChromeWindow / this = Object Window
        return this;
      }
    }
  }
  dump("\nopening the last resort window");
  newWin = (aFlags !== void(0)) ? this.__mozilla__open(aURL, aName, (this.document.defaultView.top == window.top ? aFlags.replace(/(alwaysRaised|alwaysLowered|z-lock)(=(1|0|true|false|yes|no))?,?/ig, '') || void(0) : aFlags)) : this.__mozilla__open(aURL, aName);
  return newWin;
}
////////////////////////////////////////////////////////////////////////////////
// This is called from the 'onWebProgressSTART' even handler in tabbrowser.xml
////////////////////////////////////////////////////////////////////////////////
function onWebProgressSTART(aWindow, aFollowFrameFlag)
{
  if ('frames' in aWindow && !aFollowFrameFlag) {
    for (var i = 0; i < aWindow.frames.length; i++)
      onWebProgressSTART(aWindow.frames[i]);
  }
  if ('mz_window_init' in aWindow || !aWindow.open) {
    return;
  }
  if (!('__mozilla__open' in aWindow.Window.prototype)) {
    aWindow.Window.prototype.__mozilla__open = aWindow.Window.prototype.open;
  }
  aWindow.eval('Window.prototype.open = '+this.mzWindowOpen.toSource());
  aWindow.timeStamp = (new Date()).getTime();
  aWindow.mz_window_init = true;
}
////////////////////////////////////////////////////////////////////////////////
// This is called from the 'onWebProgressSTOP' event handler in tabbrowser.xml
////////////////////////////////////////////////////////////////////////////////
function onWebProgressSTOP(aWindow)
{
  if (!('mz_window_init' in aWindow)) {
    onWebProgressSTART(aWindow, true);
  }
  if ('frames' in aWindow) {
    for (var i = 0; i < aWindow.frames.length; i++) {
      onWebProgressSTOP(aWindow.frames[i]);
    }
  }
}
////////////////////////////////////////////////////////////////////////////////
window.__mozilla__open = window.open;
if (window.open && mzPreferences.readMyPref("multizilla.window-open.redirector", "bool", false)) {
  if (!isNavigatorWindow())
    window.open = eval(mzWindowOpen.toSource());
}
