/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - October 4, 2003.
   -
   - The Initial Developer of the Original Code is HJ van Rantwijk.
   - Portions created by HJ van Rantwijk are Copyright (C) 2003-2004
   - HJ van Rantwijk.  All Rights Reserved.
   -
*/

const ADDBUTTON   = 0;
const MERGEBUTTON = 1;

const NO_ROWS_AVAILABLE      = -1;
const NO_ROWS_SELECTED       = 0;
const MIXED_ROWS_SELECTED    = 1;
const MULTIPLE_ROWS_SELECTED = 2;
const TAB_ROW_SELECTED       = 3;
const WINDOW_ROW_SELECTED    = 4;

const nsIWindowDataSource = Components.interfaces.nsIWindowDataSource; // We need this for multiviewsRDF.js!
const nsIRDFResource = Components.interfaces.nsIRDFResource;
const nsIRDFLiteral = Components.interfaces.nsIRDFLiteral;

var mzPreferences = window.opener.mzPreferences; // declared in quickPrefsMenu.js, but out of reach for us!
var pref = null; // declared in navigator.js, but out of reach for us!
var gBrowser = null; // declared in navigator.js, but out of reach for us!
var gResource = "";
var bundle;
var gSelectedWindowID = null;
var contextMenuitems = null;

var prefWindow = null;
var prefPanelNames;
var managerTypes = [ "sessionManager" ];
var dialogType = "sessionManagerPrefWindow";
var panelPath = "chrome://multiviews/content/session-manager/preferences/";
var gUseStackedWindows = false;
// Global vars used by confirmation dialog
var gConfirmCheckbox = false;
var gEscapeKeyPressed = false;
var gConfirmForID = 0;

/***
  * Toolbar button ID arrays that will be replaced with elements in updateToolbarButtons()
  * note: keep the order in-line with the corresponding XUL code!
  */
var toolbarButtons = [ "addSessionButton", "mergeTabsButton", "openSessionButton", "reloadButton", "propertiesButton", "renameButton", "deleteButton", "prefsButton" ];
/***
  * Toolbar button arrays, to ease the pain of enabling/disabling the buttons
  */
var defaultEnabledButtons      = [  true, false, false, false, false, false, false, true ];
var tabEnabledButtons          = [  true, false,  true,  true,  true, false,  true, true ];
var windowEnabledButtons       = [  true, false,  true, false, false,  true,  true, true ];
var selectionEnabledButtons    = [  true, false,  true, false, false, false,  true, true ];
var multipleRowsEnabledButtons = [  true, false,  true, false, false, false,  true, true ];

////////////////////////////////////////////////////////////////////////////////
function getNavigatorWindow(aOpenFlag)
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var navigatorWindow = windowManagerInterface.getMostRecentWindow("navigator:browser");

  if (!navigatorWindow) {
    if (!aOpenFlag)
      return null;
    else {
      window.open();
      navigatorWindow = windowManagerInterface.getMostRecentWindow("navigator:browser");
    }
  }
  return navigatorWindow;
}
////////////////////////////////////////////////////////////////////////////////
function getTabbrowser(aOpenFlag)
{
  var navigatorWindow = getNavigatorWindow(aOpenFlag);

  if (navigatorWindow)
    return navigatorWindow.getBrowser();
  return null;
}
////////////////////////////////////////////////////////////////////////////////
function getBrowser()
{
  return getTabbrowser(true);
}
////////////////////////////////////////////////////////////////////////////////
// This function is copied over from multiviewsInit.js because it is not 
// available from multiviewsRDF.js (read 'it is not included').
////////////////////////////////////////////////////////////////////////////////
function getNumberOfWindowsByType(aType)
{
  var defaultType = "navigator:browser";

  if (!aType) // default check for browser window
    aType = defaultType;

  var windowCounter = 0;
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
  var enumerator = windowManager.getEnumerator(aType);

  while (enumerator.hasMoreElements()) {
    var aWindow = enumerator.getNext();
    windowCounter++;
  }
  return windowCounter;
}
////////////////////////////////////////////////////////////////////////////////
function updateToolbarButtons()
{
  // Check if the toolbar buttons are initialized
  if (typeof toolbarButtons[0] == 'string') {
    // Not initialized yet, initialize button array
    for (var i = 0; i < toolbarButtons.length; i++)
      toolbarButtons[i] = document.getElementById(toolbarButtons[i]);
  }
  var buttons = defaultEnabledButtons;
  var treeState = sessionManager.getCurrentTreeState();

  if (treeState > NO_ROWS_SELECTED) {

    // No, check tree selection state and choose the appropriate toolbar button array
    if (treeState == TAB_ROW_SELECTED)
      buttons = tabEnabledButtons;
    else if (treeState == WINDOW_ROW_SELECTED)
      buttons = windowEnabledButtons;
    else if (treeState == MIXED_ROWS_SELECTED)
      buttons = selectionEnabledButtons;
    else if (treeState == MULTIPLE_ROWS_SELECTED)
      buttons = multipleRowsEnabledButtons;
  }
  // Enable/disable the toolbar buttons based on the toolbar array
  for (var buttonIndex = 0; buttonIndex < toolbarButtons.length; buttonIndex++)
    toolbarButtons[buttonIndex].disabled = !buttons[buttonIndex];

  toolbarButtons[ADDBUTTON].disabled = areAllTabsBlank(tabbrowser);
  toolbarButtons[MERGEBUTTON].disabled = areAllTabsBlank(tabbrowser) || treeState == NO_ROWS_SELECTED || treeState != WINDOW_ROW_SELECTED;
}
////////////////////////////////////////////////////////////////////////////////
function initContextMenu(aEvent)
{
  var i = 0;
  // Disable all menu items, but the first one, for clicking on an empty spot in the tree
  if (sessionManager.getRowNumber(aEvent) == -1) {
    var menuItems = aEvent.originalTarget.childNodes;
    // Iterate over all menu items and disable them
    for (i = 1; i < menuItems.length; i++)
      menuItems[i].setAttribute("disabled", true);
    return;
  }
  var rows = tree.treeBoxObject.view.rowCount;
  var selectedResources = sessionManager.getSelectedResources();
  // Disable "Select All" when there's a selection
  var selectAll = document.getElementById("SelectAll");
  // Disable "Deselect All" when there's no selection
  var deselectAll = document.getElementById("UndoSelection");

  if (selectedResources.length > 1) {
    selectAll.setAttribute("disabled", "true");
    deselectAll.removeAttribute("disabled");
  }
  else { // No selection, enable "Select All" and disable "Deselect All"
    if (rows > 1)
      selectAll.removeAttribute("disabled");
    deselectAll.setAttribute("disabled", "true");
  }
  var tabElements = document.getElementsByAttribute("display_for", "tabs");
  var windowElements = document.getElementsByAttribute("display_for", "windows");
  var tabAndWindowElements = document.getElementsByAttribute("display_for", "tabs-and-windows");
  var isMixed = sessionManager.checkForMixedSelection();
  var isContainer = isMixed ? false : tree.treeBoxObject.view.isContainer(sessionManager.getRowNumber(aEvent));
  var isWindow = isMixed ? false : isContainer;
  var isTab = isMixed ? false : !isWindow;

  for (i = 0; i < tabElements.length; i++) {
    tabElements[i].setAttribute("disabled", !isTab);
    tabElements[i].hidden = !isTab;
  }
  for (i = 0; i < windowElements.length; i++) {
    windowElements[i].setAttribute("disabled", !isWindow);
    windowElements[i].hidden = !isWindow;
  }
  for (i = 0; i < tabAndWindowElements.length; i++) {
    tabAndWindowElements[i].setAttribute("disabled", !isMixed);
    tabAndWindowElements[i].hidden = !isMixed;
  }
  document.getElementById('SetInterval').setAttribute("disabled", (!isTab || selectedResources.length > 1));
  document.getElementById('AddNewSession').setAttribute("disabled", areAllTabsBlank(tabbrowser));
  document.getElementById('MergeTabs').setAttribute("disabled", areAllTabsBlank(tabbrowser) || (selectedResources.length == 0) || (!isContainer));
  document.getElementById('OpenSMPropertyWindow').setAttribute("disabled", (isContainer || selectedResources.length > 1));
  document.getElementById('RenameSession').setAttribute("disabled", (!isContainer || selectedResources.length > 1));
}
//////////////////////////////////////////////////////////////////////////////
function initReloadPopup(aPopup, aEvent)
{
  if (toolbarButtons[3].disabled) {
    aEvent.preventDefault();
    return;
  }
  tabbrowser.initReloadPopup(aPopup, null, sessionManager.getReloadInterval());
}
//////////////////////////////////////////////////////////////////////////////
function openSMPropertyWindow()
{
  var treeState = sessionManager.getCurrentTreeState();

  if (treeState == NO_ROWS_SELECTED || treeState == MULTIPLE_ROWS_SELECTED || currentRow == -1)
    return;
  gResource = sessionManager.getResourceForRow(currentRow); // tab    -> urn:session-x:window-x:tabs:tab-x
                                                            // window -> urn:session-x:window-x:tabs:root
  var winFeatures = "chrome, dependent, modal, dialog" + (gUseStackedWindows ? "" : ", resizable=yes, centerscreen");
  window.openDialog("chrome://multiviews/content/session-manager/properties/sessionManagerProperties.xul", 
                    "smproperties", winFeatures, window);
}
//////////////////////////////////////////////////////////////////////////////
function openPrefOverlay()
{
  var winFeatures = "chrome, dependent, modal, dialog" + (gUseStackedWindows ? "" : ", resizable=yes, centerscreen");
  window.openDialog("chrome://multiviews/content/session-manager/preferences/sessionManagerPrefWindow.xul", 
                    "prefWindow", winFeatures, window, "");
}
//////////////////////////////////////////////////////////////////////////////
function getTargetWindow()
{
  gSelectedWindowID = null;

  window.openDialog("chrome://multiviews/content/session-manager/miscellaneous/sessionManagerSelectWindow.xul", "selectWindow",
                    "chrome, dependent, modal, dialog, centerscreen");

  if (gSelectedWindowID) {
    var windowManagerDS = Components.classes['@mozilla.org/rdf/datasource;1?name=window-mediator'].getService(nsIWindowDataSource);
    return windowManagerDS.getWindowForResource(gSelectedWindowID);
  }
  return null;
}
//////////////////////////////////////////////////////////////////////////////
function areAllTabsBlank(aTabBrowser)
{
  if (!aTabBrowser)
    aTabBrowser = getTabbrowser(true);

  var numberOfTabs = aTabBrowser.mTabContainer.childNodes.length;
  var numberOfBlankTabs = getNumberOfBlankTabs(aTabBrowser);

  return (numberOfTabs == numberOfBlankTabs) ? true : false;
}
//////////////////////////////////////////////////////////////////////////////
function getNumberOfBlankTabs(aTabBrowser)
{
  var blankTabs = 0;
  var browsers = aTabBrowser.mPanelContainer.childNodes.length;

  for (var i = 0; i < browsers; i++) {
    var browser = aTabBrowser.getBrowserAtIndex(i);
    var tab = aTabBrowser.getTabForBrowser(browser);

    if (tab.hasAttribute("busy")) {
      if (tab.getAttribute("firstPage") == "about:blank")
        blankTabs++;
    }
    else if (browser.currentURI.spec == "about:blank")
      blankTabs++;
  }
  return blankTabs;
}
//////////////////////////////////////////////////////////////////////////////
// This is a dummy function, called from the pref manager (for compatibility only)
function switchTree(gPanel)
{
  return;
}
//////////////////////////////////////////////////////////////////////////////
function switchPanel(aPanel)
{
  var categoryButton = null;
  var panelFile = null;

  prefWindow.gSecondLevelPrefs = false;

  if (aPanel == "sessionManagerPrefWindow")
    panelFile = "sessionManagerMainPanel.xul";
  prefWindow.refreshPanel(panelFile, categoryButton);
}
//////////////////////////////////////////////////////////////////////////////
function initWindowOpenType()
{
  try {
    gUseStackedWindows = pref.getBoolPref("multizilla.managers.use-stacked-windows");
  } catch(ex) {
    gUseStackedWindows = /Win/.test(navigator.platform);
  }
}
//////////////////////////////////////////////////////////////////////////////
// We need this modified copy (from browser.js) for function 'preventDupeTab'
//////////////////////////////////////////////////////////////////////////////
function BrowserReloadWithFlags(reloadFlags)
{
  var webNav = getBrowser().webNavigation;

  try {
    var sh = webNav.sessionHistory;
    if (sh)
      webNav = sh.QueryInterface(Components.interfaces.nsIWebNavigation);
  } catch(ex) {
  }
  try {
    webNav.reload(reloadFlags);
  } catch(ex) {
  }
}
////////////////////////////////////////////////////////////////////////////////
function getConfirmation(aWindow, aTitle, aText, aCheckboxText)
{
  gConfirmCheckbox = false;
  var checkbox = {};
  var checkboxText = aCheckboxText ? stringBundle.getString(aCheckboxText) : null;
  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
  var result = promptService.confirmEx(aWindow, stringBundle.getString(aTitle), stringBundle.getString(aText),
                                       (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES) + 
                                       (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_NO),
                                       null, null, null, checkboxText, checkbox);

  gConfirmCheckbox = checkbox.value;
  return (result == 0); // Yes returns 0 / No, Esc and X returns 1
}
//////////////////////////////////////////////////////////////////////////////
var sessionManager = {

  tree : null,
  tabbrowser : null,
  lastResource : null,
  reloadInterval : 0,
  currentRow : null,
  // tooltipBox : null,
  // tabNumberTooltip : null,
  // sessionNumberTooltip : null,
  // windowNumberTooltip : null,

  stringBundle : null,

  initSessionManager: function()
  {
    tree = document.getElementById("sessionManagerTree");
    initializeRDFStorageFile();
    tree.database.AddDataSource(mzRDFStore);
    tree.builder.rebuild();
    tabbrowser = getTabbrowser(true);
    gBrowser = getBrowser();
    stringBundle = document.getElementById("properties");
    // tooltipBox = document.getElementById("tooltipBox");
    // tabNumberTooltip = tooltipBox.childNodes[0];
    // sessionNumberTooltip = tooltipBox.childNodes[1];
    // windowNumberTooltip = tooltipBox.childNodes[2];
    lastResource = null;
    reloadInterval = 0;
    currentRow = -1;
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefService);
    pref = prefService.getBranch(null);

    if (!window.arguments || (window.arguments && window.arguments[1] == 'undefined' || window.arguments[1] == false)) {
      // No need to open a useless Tab Session Manager!
      if (tree.treeBoxObject.view.rowCount == 0) {
        var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
        promptService.alert(window, stringBundle.getString("mzWizard"), stringBundle.getString("tsmNoSessionsAvailable"));
        window.close();
      }
    }
    prefPanelNames = stringBundle.getString("prefPanelNames").split(',');
    sessionManager.setDefaultButton();
    updateToolbarButtons();
    sizeToContent();
    initWindowOpenType();

    if (window.arguments && window.arguments[0] == "preferences")
      setTimeout(openPrefOverlay, 0);
  },

  reInitSessionManager: function()
  {
    tree = document.getElementById("sessionManagerTree");
    tree.builder.rebuild();
    lastResource = null;
  },

  click: function(aEvent)
  {
    currentRow = this.getRowNumber(aEvent);

    if (currentRow == -1)
      return;
    reloadInterval = sessionManager.getReloadInterval(aEvent);

    if (aEvent.button == 1) // || (aEvent.button == 0 && aEvent.detail == 2))
      sessionManager.onAccept(true, false, false);
  },

  getRowNumber: function(aEvent)
  {
    var X = aEvent.pageX;
    var Y = aEvent.pageY;
    return tree.treeBoxObject.getRowAt(X, Y);
  },

  getResourceForRow: function(aRowNumber)
  {
    return (aRowNumber > -1) ? tree.builderView.getResourceAtIndex(aRowNumber).Value : null;
  },

  getResourceSpecs: function(aResource)
  {
    var numbers = aResource.replace(/[a-z:.#\/-]/g, '');
    return numbers.split('-');
  },

  getSelectedResources: function()
  {
    var resources = new Array();
    var rows = this.getSelectedRows();

    for (i in rows) {
      resources[i] = this.getResourceForRow(i);
    }
    if (resources)
      return resources;
    return null;
  },

  getSelectedRows: function()
  {
    var rows = -1;
    var selectedRows = new Array();
    // Get and return all selection ranges
    var rangeCount = tree.view.selection.getRangeCount();
    // Iterate over all ranges to collect the selected rows
    for (var range = 0; range < rangeCount; range++) {
      var startSelection = {};
      var endSelection = {};
      tree.view.selection.getRangeAt(range, startSelection, endSelection);
      // Collect all rows and store them in our array
      for (var row = startSelection.value; row <= endSelection.value; row++) {
        // Check for invalid row numbers
        if (startSelection.value != -1 || endSelection.value != -1)
          selectedRows[++rows] = row;
      }
    }
    if (selectedRows)
      return selectedRows;
    return null;
  },

  getCurrentTreeState: function()
  {
    if (tree.treeBoxObject.view.rowCount < 1)
      return NO_ROWS_AVAILABLE;

    var selectedRows = sessionManager.getSelectedRows();

    if (selectedRows.length == 0)
      return NO_ROWS_SELECTED;

    var isMixed = sessionManager.checkForMixedSelection();

    if (isMixed)
      return MIXED_ROWS_SELECTED;

    var isMultiple = (selectedRows.length > 1);

    if (isMultiple)
      return MULTIPLE_ROWS_SELECTED;

    var isContainer = (currentRow > -1) ? tree.treeBoxObject.view.isContainer(tree.currentIndex) : false;
    var isWindow = isMultiple ? false : isContainer;
    var isTab = isMultiple ? false : !isWindow;

    if (isTab)
      return TAB_ROW_SELECTED;
    else if (isWindow)
      return WINDOW_ROW_SELECTED;
    return NO_ROWS_SELECTED;
  },

  getReloadInterval: function(aEvent)
  {
    var interval = 0;
    var columnID = "sm-interval";
    columnID = ('columns' in tree) ? tree.columns[columnID] : columnID;

    if (aEvent) {
      var row = this.getRowNumber(aEvent);
      interval = (row > -1) ? tree.view.getCellText(row, columnID) : 0;
    }
    else if (!aEvent && tree.currentIndex != -1)
      interval = tree.view.getCellText(tree.currentIndex, columnID);

    if (interval)
      return interval;
    return 1;
  },

  setReloadInterval: function(aEvent, aValue)
  {
    var resource = this.getResourceForRow(currentRow);
    var intervalProp = RDF.GetResource("http://multizilla.mozdev.org/rdf#interval");
    var currentValue = mzRDFStore.GetTarget(RDF.GetResource(resource), intervalProp, true);

    if (aValue == 0) // Remove interval property from tree
      mzRDFStore.Unassert(RDF.GetResource(resource), intervalProp, currentValue);
    else
      addOrChangeValue(mzRDFStore, RDF.GetResource(resource), intervalProp, currentValue, RDF.GetLiteral(aValue));
  },

  setRowTooltip: function(aEvent)
  {
    // dump("\nlocalName: " + aEvent.originalTarget.localName);
    var row = this.getRowNumber(aEvent);
    if (row == -1)
      return;

    var resource = this.getResourceForRow(row);
    var resourceValues = this.getResourceSpecs(resource);
    resourceValues = this.convertResourceDataToNumbers(resourceValues);

    if (resourceValues.length == 3) {
      tabNumberTooltip.setAttribute("value", "Tab: " + resourceValues[2]);
      tabNumberTooltip.removeAttribute("hidden");
    }
    else
      tabNumberTooltip.setAttribute("hidden", "true");

    windowNumberTooltip.setAttribute("value", "Window: " + resourceValues[1]);
    sessionNumberTooltip.setAttribute("value", "Session: " + resourceValues[0]);
  },

  setDefaultButton: function()
  {
    var dialogButton = document.documentElement.getButton("accept");
    dialogButton.removeAttribute("default");
    dialogButton = document.documentElement.getButton("cancel");
    dialogButton.setAttribute("default", "true");
    dialogButton.focus();
  },

  convertResourceDataToNumbers: function(aDataArray)
  {
     for (i in aDataArray)
       aDataArray[i] = Number(aDataArray[i])+1;
     return aDataArray;
  },

  checkForMixedSelection: function()
  {
    var tabs = 0;
    var windows = 0;
    var mixed = false;

    // Collect selected tree rows
    var selectedItems = this.getSelectedRows();
    // Do we have any selected items?
    if (selectedItems && selectedItems.length > 0) {
      // Iterate over the seleted items
      for (var row = 0; row < selectedItems.length; row++) {
        if (tree.treeBoxObject.view.isContainer(selectedItems[row]))
          windows++;
        else
          tabs++;
        // Do we have a mixed selection?
        if (tabs > 0 && windows > 0) {
          mixed = true;
          break;
        }
      }
    }
    return mixed;
  },

  removeWindowItemsFromSelection: function()
  {
    // Collect selected tree rows
    var selectedItems = this.getSelectedRows();
    // Do we have any selected items?
    if (selectedItems) {
      // Iterate over the seleted items to clear the window items
      for (row in selectedItems) {
        if (tree.treeBoxObject.view.isContainer(row))
          tree.view.selection.clearRange(row, row);
      }
    }
  },

  selectRows: function(aCommand)
  {
    switch (aCommand) {
      case "all": // Called from the context menu, or Ctrl+A ?
           tree.view.selection.selectAll();
           this.removeWindowItemsFromSelection();
           break;
      case "none": // Called from the context menu, or Ctrl+Z ?
           tree.view.selection.clearSelection();
           tree.view.selection.select(tree.currentIndex);
           break;
    }
    tree.focus();
  },

  renameTabSession: function()
  {
    if (tree.currentIndex != -1) {
      var resource = sessionManager.getResourceForRow(tree.currentIndex);
      renameTabSession(mzRDFStore, resource);
    }
  },

  addNewSession: function()
  {
    var browserWindow, windowID, sessionID = 0;

    if (getNumberOfWindowsByType() > 1)
      browserWindow = getTargetWindow();
    else
      browserWindow = getNavigatorWindow(true);
    if (!browserWindow)
      return;
    windowID = getRelativeWindowResourceID(browserWindow);
    sessionID = getNextSessionID(sessionID, windowID);

    var resource = "urn:" + sessionID + ":" + windowID + ":tabs:root";
    storeGroupData(mzRDFStore, resource, browserWindow.gBrowser);
    tree.builder.rebuild();
  },

  mergeTabs: function()
  {
    var browserWindow, resource = this.getResourceForRow(currentRow);
    resource = mzGetLastResourceFromContainer(mzRDFStore, resource);
    var tabID = this.getResourceSpecs(resource);
    tabID++;

    if (getNumberOfWindowsByType() > 1)
      browserWindow = getTargetWindow();
    else
      browserWindow = getNavigatorWindow(true);
    if (!browserWindow)
      return;
    addTabDataSeqToDS(mzRDFStore, resource, tabID, browserWindow.gBrowser, true);
  },

  doConfirm: function(additionalText)
  {
    var dialogTitle = stringBundle.getString("tsmConfirmActionTitle");
    var dialogText = stringBundle.getString("tsmConfirmActionText");
    additionalText = stringBundle.getString(additionalText);
    dialogText = dialogText.replace(/%S/, additionalText);
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
    var result = promptService.confirmEx(window, dialogTitle, dialogText,
                                         (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES) + 
                                         (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_NO),
                                         null, null, null, null, {value:0});
    tree.focus();
    return (result == 0); // Yes returns 0 / No, Esc and X returns 1
  },

  onSelect: function()
  {
    currentRow = tree.currentIndex;
    document.documentElement.getButton("accept").removeAttribute("disabled");
    updateToolbarButtons();
  },

  onRemove: function(skipLastResourceCheck)
  {
    var rows = this.getSelectedRows();
    var confirmActionText = (rows.length > 1) ? "tsmRemoveMultipleItems" : "tsmRemoveSingleItem";

    if (!this.doConfirm(confirmActionText))
      return;

    if (rows && rows.length) {
      // Delete the selected row(s from the bottom to the top)
      for (var i = rows.length-1; i > -1; i--) {
        /***
          * 'urn:session-x:window-x:tabs:root' for windows (containers)
          * 'urn:session-x:window-x:tabs:tab-x' for tabs (normal rows)
          */
        var resource = this.getResourceForRow(rows[i]);
        // Prevent duplicated actions!
        if (!skipLastResourceCheck && (lastResource == null || lastResource == resource))
          break;
        if (tree.treeBoxObject.view.isContainer(rows[i])) // Window
          removeAllResourcesFor(mzRDFStore, resource);
        else // Single Tab
          removeTabResourcesFor(mzRDFStore, resource);
      }
    }
    this.reInitSessionManager();
  },

  onHelp: function(aPageAnchor)
  {
    var pageAnchor = "";
    var helpPages = stringBundle.getString("helpPages-SessionManager");

    if (aPageAnchor) {
      if (typeof aPageAnchor == 'string')
        pageAnchor = "#" + aPageAnchor;
      else // event
        pageAnchor = "#" + aPageAnchor.target.id.replace(/^sessionManager-/, '');
    }
    // We could eventually check the on-line state!
    tabbrowser = getTabbrowser(true);
    tabbrowser.selectedTab = tabbrowser.addOrGetTab(helpPages + pageAnchor);
    window.close();
  },

  onCancel: function()
  {
    var retValue = this.confirmClose(gConfirmForID);
    return retValue;
  },

  onAccept: function(skipLastResourceCheck, closeSessionManager, openInNewWindow)
  {
    var rows = this.getSelectedRows();
    var result;

    if (rows && rows.length) {
      mzTabIndex = 0;
      var startupFlag = false;
      // [Open] openInNewWindow = true [click] openInNewWindow = false!
      closeSessionManager = openInNewWindow ? true : false;
      // check for open browser window
      if (getNavigatorWindow(false) == null) { // no open browser window
        tabbrowser = getTabbrowser(true); // open a new browser window and re-initialize 'tabbrowser'
        openInNewWindow = false;
        startupFlag = true;
      }
      for (i in rows) {
        var resource = this.getResourceForRow(rows[i]);
        // Prevent duplicated actions!
        if (!skipLastResourceCheck && (/* lastResource == null || */ lastResource == resource)) {
          continue;
        }
        if (tree.treeBoxObject.view.isContainer(rows[i])) { // Restore Window(s)
          // Do we really need to open e new window?
          openInNewWindow = !areAllTabsBlank(tabbrowser);
          // Open new browser window if the user clicked on the Ok button, or 
          // when he tries to fool us, by closing all available browser windows!
          var browser = openInNewWindow ? openWindowAndReturnBrowser() : tabbrowser;
          result = restoreTabsFromResource(mzRDFStore, browser, resource, false, true);
        }
        else { // Restore Tab(s)
          var newTab = (mzTabIndex > 0 || rows.length == 1) ? true : false;
          startupFlag = newTab;
          mzTabIndex++;
          result = restoreTabFromResource(mzRDFStore, tabbrowser, RDF.GetResource(resource), newTab, startupFlag);
        }
        if (result == -1)
          break;
      }
    }
    if (closeSessionManager)
      window.close();
  },

  confirmClose: function(aConfirm)
  {
    var shouldClose = true;
    var confirmPrefID = "multizilla.session-manager.confirm.quit";
    var dialogCheckboxText, confirmPrefValue = 15;
    /* ::::
       1  = Close Button
       2  = ESC key pressed
       4  = Ctrl-W
       8  = Click on [X] on Window
    :::: */
    try {
      // try to read the pref
        confirmPrefValue = pref.getIntPref(confirmPrefID);
    } catch(ex) {
      // Yikes, no pref found, add it with the default value
      pref.setIntPref(confirmPrefID, confirmPrefValue);
    }
    // Is the pref set for confirmation?
    if (confirmPrefValue && (confirmPrefValue & aConfirm) == aConfirm) {
      // Yes, ask the users confirmation
      switch(aConfirm) {
        case (aConfirm & 1): dialogCheckboxText = 'managerSkipConfirmForCloseButton';
          break;
        case (aConfirm & 2): dialogCheckboxText = 'managerSkipConfirmForEscapeKey';
          break;
        case (aConfirm & 4): dialogCheckboxText = 'managerSkipConfirmForCtrlW';
          break;
        case (aConfirm & 8): dialogCheckboxText = 'managerSkipConfirmForWindowX';
          break;
      }
      shouldClose = getConfirmation(window, 'tmConfirmCloseTitle', 'tsmConfirmCloseText', dialogCheckboxText);
      // Is the checkbox checked and do we have a pref value to work with?
      if (gConfirmCheckbox && (confirmPrefValue >= aConfirm)) {
        // Yes, update pref
        pref.setIntPref(confirmPrefID, (confirmPrefValue -= aConfirm));
      }
    }
    return shouldClose;
  },

  onDragStart: function(aEvent, aXferData, aDragSession)
  {
    if (tree.treeBoxObject.view.rowCount > 0) {
      var resources = this.getSelectedResources();
      var resource;
      var resourceList = "";

      for (i in resources) {
        resource = resources[i];
        resourceList = resourceList.concat((i == 0 || i == resources.length) ? resource : ","+resource);
      }
      aXferData.data = new TransferData();
      aXferData.data.addDataForFlavour("text/x-moz-url", "sm:resources:"+resourceList);

      /* var X = aEvent.pageX;
      var Y = aEvent.pageY;
      var dragStartRow = tree.treeBoxObject.getRowAt(X, Y);

      if (dragStartRow > -1) {
        var resource = this.getResourceForRow(dragStartRow);
        aXferData.data = new TransferData();
        aXferData.data.addDataForFlavour("text/x-moz-url", "sm:resources:"+resource);
      } */
      // dump("\nSM-resource:");
    }
  },

  onDragOver: function(aEvent, aXferData, aDragSession)
  {
    var X = aEvent.pageX;
    var Y = aEvent.pageY;
    var row = tree.treeBoxObject.getRowAt(X, Y);
    dump("\nSM-row: "+row);
  },

  onDrop: function(aEvent, aXferData, aDragSession)
  {
    dump("\nSM-onDrop: "+aEvent.originalTarget.nodeName);
  },

  onDragExit: function(aEvent, aXferData, aDragSession)
  {
    // dump("\nSM-onDragExit");
    // tree.childNodes[1].removeAttribute("dragover");
  },

  getSupportedFlavours: function ()
  {
    // we might want to expand this
    var flavourSet = new FlavourSet();
    flavourSet.appendFlavour("text/unicode");
    return flavourSet;
  },

  canDrop: function (aEvent, aDragSession)
  {
    // dump("\nSM-canDrop");
    var X = aEvent.pageX;
    var Y = aEvent.pageY;
    var row = tree.treeBoxObject.getRowAt(X, Y);

    if (row > -1 && tree.treeBoxObject.view.isContainer(row)) {
      aDragSession.canDrop = true;
      return true;
    }
    else 
      return false;
    /* allow drops from Tab Session Manager only!
    var doc = aDragSession.sourceNode.ownerDocument;
    var elem = doc.getElementById("sessionManagerWindow");
    return (elem && (elem.getAttribute("windowtype") == "multizilla:sessionmanager")); */
  }
};

addEventListener("load", sessionManager.initSessionManager, false);