/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - November 20, 2003.
   -
   - The Initial Developer of the Original Code is HJ van Rantwijk.
   - Portions created by HJ van Rantwijk are Copyright (C) 2003-2004
   - HJ van Rantwijk.  All Rights Reserved.
   -
*/

const MZ_TRUE = "true";
const MZ_FALSE = "false";

var gDialogManager, gHeight, gTabbrowser, gModified;
var gBackLabel, gBackImage, gForwardLabel, gForwardImage;
var reloadInterval, helpPages, dataCollection = null;
var elementArray = new Array();
var elementIDs = new Array("label", "custom", "firstpage", "image", "location", "interval", 
                           "selected", "ordinal", "allowPlugins", "allowJavascript", 
                           "allowMetaRedirects", "allowSubframes", "allowImages", 
                           "interval-enabled", "allow-all", "suppress-all");

var defaultValues = ["about:blank", "false", "about:blank", "", "about:blank", "0", "false", "true", "true", "true", "true", "true"];

function Startup()
{
  gDialogManager = window.arguments[0];
  gHeight = (gDialogManager.innerHeight / 50 - 2);

  document.getElementById('mac_button_spring').hidden = !(/Mac/.test(navigator.platform));

  if (gDialogManager.gUseStackedWindows) {
    document.documentElement.setAttribute("hidechrome", "true");
    window.outerWidth = gDialogManager.innerWidth;
    window.outerHeight = 0;
    slideOpen(0);
  }
  else {
    window.outerWidth = gDialogManager.outerWidth;
    window.outerHeight = gDialogManager.outerHeight;
    document.getElementById("main-box").collapsed = false;
  }
  gTabbrowser = gDialogManager.getTabbrowser(true);
  gBackImage = document.getElementById('BackImage');
  gBackLabel = document.getElementById('BackLabel');
  gForwardLabel = document.getElementById('ForwardLabel');
  gForwardImage = document.getElementById('ForwardImage');

  // setResource(0);
  // initTabProperties();
  // selectDefaultButton();
  refreshPropertyData(0)
}

function slideOpen(aStep)
{
  if (aStep < gHeight) {
    window.outerHeight += 50;
    setTimeout(slideOpen, 20, ++aStep);
  } 
  else {
    /* var defaultHeight = gDialogManager.document.documentElement.getAttribute('inner-height');

    if (gDialogManager.innerHeight < defaultHeight)
      gDialogManager.innerHeight = defaultHeight; */
    window.outerHeight = gDialogManager.innerHeight;
    document.getElementById("main-box").collapsed = false;
  }
}

function slideClosed(aStep)
{
  document.getElementById("dialogButtonBox").collapsed = true;

  if (gDialogManager.gUseStackedWindows && aStep < gHeight) {
    window.outerHeight -= 50;
    setTimeout(slideClosed, 10, ++aStep);
  } 
  else
    window.close();
}

function onCancel()
{
  slideClosed(0);
}

function onAccept()
{
  if (mzRDFStore) // Tab Session Manager needs to update multiviews.rdf
    storeDataCollection();
  slideClosed(0);
}

function initTabProperties()
{
  dump("\ninitTabProperties()");
  // Is this the first run?
  if (typeof elementIDs[0] == 'string') {
    // Yes, initialize element array (used in activateSettings)
    for (i in elementIDs)
      elementArray[i] = document.getElementById(elementIDs[i]);
  }
  // Initialize textboxes, checkboxes and buttons
  // dump("\ngDialogManager.gResource: " + gDialogManager.gResource);
  dataCollection = getTabDataFromResource(gDialogManager.dataSource, RDF.GetResource(gDialogManager.gResource));

  if (dataCollection) {
    activateSettings(dataCollection);
    // Initialize the 'Allow All' or 'Suppress All' buttons
    onProperty(null);
  }
  setBackForwardButtons();
}

function activateSettings(aDataCollection)
{
  dump("\nactivateSettings()");
  elementArray[0].value = aDataCollection[0].value; // title
  elementArray[1].checked = (aDataCollection[1].value == MZ_TRUE); // custom
  elementArray[2].value = aDataCollection[2].value; // firstpage
  elementArray[3].value = aDataCollection[3].value; // image
  elementArray[4].value = aDataCollection[4].value; // location
  reloadInterval = aDataCollection[5].value; // interval
  elementArray[6].checked = (aDataCollection[6].value == MZ_TRUE) ? false : true;
  // elementArray[7].value = aDataCollection[7].value; // ordinal
  elementArray[13].checked = (reloadInterval > 0);
  elementArray[5].disabled = (reloadInterval == 0);
  gTabbrowser.initReloadPopup(elementArray[5].firstChild, null, reloadInterval);
  // Tab Properties
  for (var i = 8; i < 13; i++) {
    elementArray[i].checked = (aDataCollection[i].value == MZ_TRUE);
  }  
}

function refreshReloadState()
{
  var disabled = !elementArray[13].checked
  elementArray[5].disabled = disabled;

  if (disabled) {
    elementArray[5].value = 0;
    reloadInterval = 0;
  }
}

function onProperty(aElement)
{
  var id = aElement ? aElement.getAttribute("id") : "";
  var properties = 0;
  // Count number of checked property checkboxes
  for (var i = 8; i < 13; i++) {
    if (elementArray[i].checked)
      properties++
  }
  // Called from the 'Allow All' or 'Suppress All' buttons
  if (id == elementIDs[14] || id == elementIDs[15]) {
    properties = (id == elementIDs[14]) ? 5 : 0;
    for (i = 8; i < 13; i++) {
      elementArray[i].checked = properties ? true : false;
      // Update all docShell properties
      gDialogManager.tabManager.updateTab(elementIDs[i], properties ? true : false);
    }
  }
  // Update button state
  elementArray[14].disabled = (properties == 5) ? true : false;
  elementArray[15].disabled = (properties == 0) ? true : false;
  // Update the docShell property
  if (id.indexOf('allow') == 0) {
    gDialogManager.tabManager.updateTab(id, aElement.checked);
  }
}

function setReloadInterval(aMenuitem)
{
  reloadInterval = (aMenuitem.getAttribute('value') * 1000);
}

function setCustomLabel(setCustomLabel)
{
  gDialogManager.tabManager.updateTab('customlabel', setCustomLabel);
}

function selectDefaultButton()
{
  window.focus();
  dump("\nselectDefaultButton()");
  var acceptButton = document.documentElement.getButton('accept');
  var cancelButton = document.documentElement.getButton('cancel');
  var defaultButton = cancelButton;

  acceptButton.removeAttribute('default');
  cancelButton.removeAttribute('default');

  if (gModified)
    defaultButton = acceptButton;
  else
    acceptButton.disabled = true;
  defaultButton.setAttribute('default', 'true');
  defaultButton.focus();
}

function setBackForwardButtons()
{
  dump("\nsetBackForwardButtons()");

  var tree = gDialogManager.tree;
  var row = tree.currentIndex;
  var rows = (tree.treeBoxObject.view.rowCount-1);
  var disabled = (row > 0) ? tree.treeBoxObject.view.isContainer(row - 1) : false;

  // dump("\nBack disabled: " + disabled);
  gBackLabel.disabled = disabled;
  gBackImage.setAttribute('disabled', disabled ? 'true' : 'false');
  disabled = (row == rows) ? true : ((row + 1) < rows) ? tree.treeBoxObject.view.isContainer(row + 1) : false;
  // dump("\nForward disabled: " + disabled);
  gForwardLabel.disabled = disabled;
  gForwardImage.setAttribute('disabled', disabled ? 'true' : 'false');
  window.focus();
}

function setResource(aDirection)
{
  dump("\nsetResource(0)");

  var tree = gDialogManager.tree;
  tree.currentIndex = tree.currentIndex + aDirection;
  // dump("\nsetResource(1)");
  gDialogManager.tabManager.selectRows('none');
  // dump("\nsetResource(2)");
  tree.treeBoxObject.ensureRowIsVisible(tree.currentIndex);
  // dump("\nsetResource(3)");
  var resource = tree.builderView.getResourceAtIndex(tree.currentIndex).Value;
  // dump("\nresource: " + resource);
  document.getElementById('TabSessionID').value = resource;
  gDialogManager.gResource = resource;
}

function refreshPropertyData(aDirection)
{
  dump("\n\nrefreshPropertyData(1)");
  // We need to check the disable state of the 'label' buttons!
  var canBack = gBackLabel.disabled;
  var canForward = gForwardLabel.disabled;

  if (aDirection == -1 && canBack || aDirection == 1 && canForward)
    return;
  // dump("\nrefreshPropertyData(2)");
  var shouldSkip = false;
  var prefString = "multizilla.tabs.session-property-changes.confirmation";

  try {
    shouldSkip = gDialogManager.pref.getBoolPref(prefString);
  } catch (ex) {
    gDialogManager.pref.setBoolPref(prefString, shouldSkip);
  }
  var result = (gModified && shouldSkip) ? 0 : 1;

  if (gModified && !shouldSkip) {
    var dialogTitle = gDialogManager.stringBundle.getString("tsmConfirmActionTitle");
    var dialogText = gDialogManager.stringBundle.getString("tsmConfirmStorageText");
    var checkboxText = gDialogManager.stringBundle.getString("tsmKeepStorageDecisionText");
    var checkbox = {checked:true}
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
    result = promptService.confirmEx(window, dialogTitle, dialogText,
                                     (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES) + 
                                     (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_NO),
                                     null, null, null, checkboxText, checkbox);

    gDialogManager.pref.setBoolPref(prefString, checkbox.value);
  }
  // if (result == 0) // Yes returns 0 and No, Esc and X on window returns 1
    // storeDataCollection();
  // dump("\nrefreshPropertyData(3)");
  setResource(aDirection);
  // dump("\nrefreshPropertyData(4)");
  gModified = false;
  initTabProperties();
  // dump("\nrefreshPropertyData(5)");
  selectDefaultButton();
  // dump("\nrefreshPropertyData(6)");
}

function getDataCollection()
{
  dump("\ngetDataCollection()");
  var defaultScheme = "http://";
  var aDataCollection = new Array();
  // Data Collection buildup
  aDataCollection[0] = {value:elementArray[0].value}; // title
  aDataCollection[1] = {value:elementArray[1].checked ? MZ_TRUE : MZ_FALSE}; // custom
  // Check for missing scheme in 'firstpage' textbox.
  if (elementArray[2].value.indexOf(":") < 0) // Add default scheme
    elementArray[2].value = defaultScheme + elementArray[2].value;
  if (elementArray[2].value == defaultScheme)
    elementArray[2].value = defaultValues[2];
  aDataCollection[2] = {value:elementArray[2].value}; // firstpage
  aDataCollection[3] = {value:elementArray[3].value}; // image
  // Check for missing scheme in 'location' textbox.
  if (elementArray[4].value.indexOf(":") < 0) // Add default scheme
    elementArray[4].value = defaultScheme + elementArray[4].value;
  if (elementArray[4].value == defaultScheme)
    elementArray[4].value = defaultValues[4];
  aDataCollection[4] = {value:elementArray[4].value}; // location
  aDataCollection[5] = {value:reloadInterval}; // Reload interval
  aDataCollection[6] = {value:elementArray[6].checked ? MZ_FALSE : MZ_TRUE};
  // aDataCollection[7] = {value:elementArray[7].value}; // ordinal
  // Tab Properties
  for (var i = 8; i < 13; i++) {
    aDataCollection[i] = {value:elementArray[i].checked ? MZ_TRUE : MZ_FALSE};
  }
  return aDataCollection;
}

function storeDataCollection()
{
  var MZ = "http://multizilla.mozdev.org/rdf#";
  var modifiedDataCollection = getDataCollection();
  var resource = RDF.GetResource(gDialogManager.gResource);

  try {
    for (var i = 0; i < 13; i++) {
      if (i == 7) // Skip ordinal
        continue;
      if (dataCollection[i].value != modifiedDataCollection[i].value) {
        var property = RDF.GetResource(MZ+elementIDs[i]);
        var newValue = RDF.GetLiteral(modifiedDataCollection[i].value);
        var currentValue = gDialogManager.dataSource.GetTarget(resource, property, true);

        if (modifiedDataCollection[i].value == defaultValues[i])
          gDialogManager.dataSource.Unassert(resource, property, currentValue);
        else
          addOrChangeValue(gDialogManager.dataSource, resource, property, currentValue, newValue);
      }
    }
  } catch(ex) { 
    // die silently
  }
  mzFlushDataSource(gDialogManager.dataSource);
  gModified = false;
}


